@@ -53,7 +53,7 @@ sub new {
     $self->{fragments} = [];
 
     $self->{max_fragments_amount} ||= 128;
-    $self->{max_payload_size}     ||= 65536;
+    $self->{max_payload_size}     ||= 65536 unless exists $self->{max_payload_size};
 
     return $self;
 }
@@ -174,7 +174,7 @@ sub next_bytes {
             $offset += 8;
         }
 
-        if ($payload_len > $self->{max_payload_size}) {
+        if ($self->{max_payload_size} && $payload_len > $self->{max_payload_size}) {
             $self->{buffer} = '';
             die "Payload is too big. "
               . "Deny big message ($payload_len) "
@@ -246,16 +246,22 @@ sub to_bytes {
         return "\x00" . $self->{buffer} . "\xff";
     }
 
-    if (length $self->{buffer} > $self->{max_payload_size}) {
+    if ($self->{max_payload_size} && length $self->{buffer} > $self->{max_payload_size}) {
         die "Payload is too big. "
           . "Send shorter messages or increase max_payload_size";
     }
 
+
+    my $rsv_set = 0;
+    if($self->{rsv} && ref($self->{rsv}) eq 'ARRAY') {
+    	for my $i (0..@{$self->{rsv}}-1) { 
+    		$rsv_set += $self->{rsv}->[$i] * (1 << (6 - $i));
+    	}
+    }
+    
     my $string = '';
-
     my $opcode = $self->opcode;
-
-    $string .= pack 'C', ($opcode + ($self->fin ? 128 : 0));
+    $string .= pack 'C', ($opcode | $rsv_set | ($self->fin ? 128 : 0));
 
     my $payload_len = length($self->{buffer});
     if ($payload_len <= 125) {
@@ -312,6 +318,12 @@ sub _mask {
     return $payload;
 }
 
+sub max_payload_size {
+    my $self = shift;
+
+    return $self->{max_payload_size};
+}
+
 1;
 __END__
 
@@ -444,4 +456,9 @@ Return the next message as is.
 
 Construct a WebSocket message.
 
+=head2 C<max_payload_size>
+
+The maximum size of the payload. You may set this to C<0> or C<undef> to disable
+checking the payload size.
+
 =cut
